# -*- coding: utf-8 -*-

import numpy as np
import matplotlib.pyplot as plt

def trapezoid(f, a, b, N):
    """Approximiert das Integral von `f` mit der Trapezregel.

    Input:
           f : Funktion f(x) welche integiert werden soll.
        a, b : untere/obere Grenze des Integrals.
           N : Anzahl Teilintervalle in der zusammengesetzten
               Trapezregel.
    """
    # TODO implementieren Sie die Trapezregel.

    I = 0.0
    return I


def simpson(f, a, b, N):
    """Approximiert das Integral von `f` mit der Simpsonregel.

    Input:
           f : Funktion f(x) welche integiert werden soll.
        a, b : untere/obere Grenze des Integrals.
           N : Anzahl Teilintervalle in der zusammengesetzten
               Simpsonregel.
    """

    # TODO implementieren Sie die Simponregel.

    I = 0.0
    return I



def quadrature_error(quadrature_rule, f, exact):
    """Berechnet den Fehler und die Konvergenzrate.

    Input:
        quadrature_rule : Quadraturformel für welche verwendet werden soll. Die
                          Funktion hat die gleiche Signature wie `simpson` und
                          `trapezoid`.

                      f : Funktion welche integriert werden soll. In der Aufgabe
                          heissen sie `f1` und `f2`.

                  exact : Exakte Referenzlösung des Integrals.

    Output:
                 errors : Fehler der Quadraturformel mit `n_chunks` Teilintervallen.
               n_chunks : Anzahl Teilintervalle.
    """

    n_chunks = 2**np.arange(3, 10)
    errors = np.empty(n_chunks.shape)  # Vorsicht, n_chunks.dtype ist ein Array
                                       # voller `int`. Deswegen kein `empty_like`.

    for i, N in enumerate(n_chunks):
        # TODO berechnen Sie den Fehler.
        errors[i] = 1.0

    return errors, n_chunks

def plot_convergence(n_chunks, errors_tr, errors_si, title):
    """Plottet einen Konvergenzplot in einem neuen Fenster.

    Der Vergleich zwischen Simpson- und Trapezregel soll 'fair' sein. Die
    Simpsonregel wertet `f` fast 2x öfter aus. Plotten Sie deswegen den Fehler
    gegen die Anzahl Funktionsaufrufe.

    Trendlinien der Ordnung 1.5, 2 und 4 werden ebenfalls geplottet.

    Input:
        n_chunks : Anzahl Teilintervalle.
       errors_tr : Fehler der Trapezregel.
       errors_si : Fehler der Simpsonregel.
           title : Titel des Plots.
    """

    n_tr = n_chunks + 1
    n_si = 2*n_chunks + 1

    plt.figure(figsize=(12,8))

    # TODO plotten Sie den Fehler in einem log-log plot.

    # TODO plotten Sie im gleichen log-log plot die Trendlinien.

    plt.grid(True)
    plt.xlabel("Anzahl Auswertungen von $f$")
    plt.ylabel("Absoluter Fehler")
    plt.legend(loc="lower left")
    plt.title(title)

def f1(x):
    return 1.0 / (1.0 + 5.0*x**2)

def f2(x):
    return np.sqrt(x)

def ex1_c():
    # Plot
    t = np.linspace(0.0, 1.0, 200)
    plt.figure(figsize=(12,8))
    plt.plot(t, f1(t), label=r"$f_1(x) = \frac{1}{1 + 5x^2}$")
    plt.plot(t, f2(t), label=r"$f_2(x) = \sqrt{x}$")
    plt.grid(True)
    plt.legend(loc="center right")
    plt.savefig("img/funktionen.pdf")
    plt.savefig("img/funktionen.png")

    # Konvergenzplot für `f1`.
    I1ex = np.arctan(np.sqrt(5.0)) / np.sqrt(5.0)

    errors_tr, n_chunks = quadrature_error(trapezoid, f1, I1ex)
    errors_si, _ = quadrature_error(simpson, f1, I1ex)

    plot_convergence(n_chunks, errors_tr, errors_si, "Konvergenz für $f_1$")
    plt.savefig("img/konvergenz_f1.png")
    plt.savefig("img/konvergenz_f1.pdf")

    # Konvergenzplot für `f2`.
    I2ex = 2.0 / 3.0

    errors_tr, n_chunks = quadrature_error(trapezoid, f2, I2ex)
    errors_si, _ = quadrature_error(simpson, f2, I2ex)

    plot_convergence(n_chunks, errors_tr, errors_si, "Konvergenz für $f_2$")
    plt.savefig("img/konvergenz_f2.png")
    plt.savefig("img/konvergenz_f2.pdf")

if __name__ == "__main__":
    ex1_c()
    plt.show()
