# -*- coding: utf-8 -*-

import numpy as np
import matplotlib.pyplot as plt

from quadrature import simpson, trapezoid, quadrature_error

def golub_welsch(n):
    """Berechnet die Knoten und Gewichte für Gauss-Legendre Quadratur.
    """
    return zeros(n-1), zeros(n-1)

def gauss_legendre(f, a, b, n):
    """ Gauss-Legendre Quadratur (nicht zusammengesetzt).

    f:     Funktion f(x)
    a, b:  Obere und untere Grenze des Intervalls.
    n:     Anzahl Quadraturpunkte.
    """

    return 0

def composite_legendre(f, a, b, N, n):
    """ Zusammengesetzte Gauss-Legendre Quadratur.

    f:     Funktion f(x)
    a, b:  Obere und untere Grenze des Intervalls.
    N:     Anzahl Teilintervalle.
    n:     Anzahl Quadraturpunkte pro Teilintervall.
    """

    return 0



def legendre_error(f, exact):
    return quadrature_error(gauss_legendre, f, exact)

def composite_legendre_error(f, exact):
    n = 4

    rule = lambda f, a, b, N: composite_legendre(f, a, b, N, n)
    errors, n_chunks = quadrature_error(rule, f, exact)

    return errors, n*n_chunks

def plot_convergence(n_evals, errors, labels, title):
    """Plottet einen Konvergenzplot in einem neuen Fenster.

    Der Vergleich zwischen Simpson- und Trapezregel soll 'fair' sein. Die
    Simpsonregel wertet `f` fast 2x öfter aus. Plotten Sie deswegen den Fehler
    gegen die Anzahl Funktionsaufrufe.

    Trendlinien der Ordnung 1.5, 2 und 4 werden ebenfalls geplottet.

    Input:
        n_evals : list of array
                  Anzahl Funktionsaufrufe pro Regel pro Auflösung.

         errors : list of array
                  Fehler pro Quadraturregel pro Auflösung.

         labels : list. Text der in der Legende erscheint.
          title : Titel des Plots.
    """

    plt.figure(figsize=(12,8))
    # TODO plotten Sie den Fehler in einem log-log plot.

    # TODO plotten Sie im gleichen log-log plot die Trendlinien.

    plt.grid(True)
    plt.xlabel("Anzahl Auswertungen von $f$")
    plt.ylabel("Absoluter Fehler")
    plt.legend(loc="lower left")
    plt.title(title)

def f1(x):
    return 1.0 / (1.0 + 5.0*x**2)

def f2(x):
    return np.sqrt(x)

def convergence_experiment(f, exact, title, filename):
    errors_tr, n_chunks = quadrature_error(trapezoid, f, exact)
    n_tr = n_chunks + 1
    errors_si, n_chunks = quadrature_error(simpson, f, exact)
    n_si = 2*n_chunks + 1

    errors_gl, n_gl = legendre_error(f, exact)
    errors_cg, n_cg = composite_legendre_error(f, exact)

    n_evals = [n_tr, n_si, n_gl, n_cg]
    errors = [errors_tr, errors_si, errors_gl, errors_cg]
    labels = ["Trapez", "Simpson", "Gauss-Legendre", "Zusammengesetzte Gauss Legende"]

    plot_convergence(n_evals, errors, labels, title)
    plt.savefig(filename + ".png")
    plt.savefig(filename + ".pdf")


def ex1_c():
    # Konvergenzplot für `f1`.
    I1ex = np.arctan(np.sqrt(5.0)) / np.sqrt(5.0)
    title = r"Quadraturfehler für $f_1(x) = \frac{1}{1 + 5x^2}$"
    convergence_experiment(f1, I1ex, title, "img/convergence_f1")

    # Konvergenzplot für `f2`.
    I2ex = 2.0 / 3.0
    title = r"Quadraturfehler für $f_2(x) = \sqrt{x}$"
    convergence_experiment(f2, I2ex, title, "img/convergence_f2")

if __name__ == "__main__":
    ex1_c()
    plt.show()
